' Get all parameters -- settings, and hook-specific values
' Note: settings will always be turned into initialLowerCamelCase even if they are
' specified as "Initial Lower Camel Case" in the info.plist file.
dim pUserName as string = Setting("username")
dim pToken as string = Setting("token")
dim pCompanyId as string = Setting("companyId")
dim pUserId as string = Setting("userId")
dim pTaskId as string = Setting2("taskId", "26-iIx--hnUw")
dim pStartDate as string = Param("startDate")
dim pEndDate as string = Param("endDate")


if pUserName = "" then
	MsgBox("Username must be set in settings")
    ScriptComplete(-1)
	return
end if

if pCompanyId = "" then
	MsgBox("Company Id must be set in settings")
    ScriptComplete(-1)
	return
end if

if pToken = "" then
	MsgBox("Token must be set in settings")
	ScriptComplete(-1)
    return
end if

if pUserId  = "" then
	MsgBox("User Id must be set in settings")
    ScriptComplete(-1)
	return
end if

if pStartDate  = "" or pEndDate = "" then
	MsgBox("startDate or endDate are missing from timesheet:publish hook call." + EOL + EOL + _
		"This bundle script may need to be updated. Please contact us at support@nearnorthsoftware.com")
    ScriptComplete(-1)
	return
end if

' Create constants for fixed objects that we always use
' Dicts
#declare kExportEntries 0
#declare kJobs 1
' Http
#declare kHttp 0
' Json
#declare kApiResult 0
' We only use one iterator at a time so we only need this one
#declare kIter 0

' Macros to check for errors from API calls that return an obj number (which may be zero)
#declare [[ if(0 > 
#declare ]].Check() ) then return MsgBox2("Script trapped an error, cannot continue")

' Open rest device using the user-specified resource as a base URL
call HttpCreateFromSetting(kHttp, "baseURL")

' Request headers are cached by the Rest device and used for all requests
HttpSetRequestHeader(kHttp, "Authorization", "Basic " + Base64(pUserName + ":" + pToken))
HttpSetRequestHeader(kHttp, "Content-Type", "application/json")

' Get list of jobs from ClickTime for mapping of job name to ClickTime JobID
call DictionaryCreate(kJobs)
dim jobsJson as string
dim publishResult as integer = 0

jobsJson = HttpDoGetRequest(kHttp, "Companies/" + pCompanyId + "/Users/" + pUserId + "/Jobs", 10)
'Log("ClickTime jobs: " + jobsJson)
call JsonParse(kApiResult, jobsJson)
'Log("JSON out: " + JsonToString(kApiResult))
Log("ClickTime Job Count: " + Str(JsonCount(kApiResult)))
dim count as integer = 0
'Log("Job name in nnScreenshots must exactly match one of these names to be imported")
for jobIdx as integer = 0 to JsonCount(kApiResult)-1
	count = count + 1
	'Log("== " + Str(count) + " ==")
	dim jobJson as integer = JsonGetElementAt(kApiResult, jobIdx)
	dim jobName as string = JsonGetString(jobJson, "DisplayName")
	dim jobId as string = JsonGetString(jobJson, "JobID")
'	Log(jobName)
	DictionarySetString(kJobs, jobName, jobId) 
	JsonClose(jobJson)
next
'Log("== End of Job List ==")
JsonClose(kApiResult)

' Create a list of all current entries we have locally, in the format used by ClickTime
' {
'	"BreakTime":0.00,
'	"Comment":"I worked on all the things.",
'	"Date":"20150425",
'	"Hours":1.00,
'	"ISOEndTime":"12:00:00",
'	"ISOStartTime":"11:00:00",
'	"JobID":"2G4TAC74WZp",
'	"PhaseID":"2IMbLnQxpII8",
'	"SubPhaseID:null,
'	"TaskID":"2S9YciWDlopY"
' }									
call DictionaryCreate(kExportEntries)
call FirstRecord()
dim activityErrors as string
for i as integer = 0 to RecordCount()-1
	dim jobId as string = TranslateJob(Val(FieldValue("activityId")))
    dim comments as string = FieldValue("entry")
    dim entryDateTime as string = FieldValue("dateTime")
    dim dateArr() as string = Split(entryDateTime, " ")
    dim entryDate as string = ReplaceAll(dateArr(0), "-", "")
    dim duration as string = Format(Val(FieldValue("duration")), "##.00")
    dim startTime as string = dateArr(1)
    dim endTime as string = TimeAdd(startTime, FieldValue("duration")+" hours")
    if TimeAddOverflowed then
        ' If the time overflowed (e.g. the startTime + this number of hours went into the next day)
        ' Then instead calculate from an endTime backwards. Just need to generate a valid startTime+endTime = duration window
        ' for clicktime to accept it.
        endTime = dateArr(1)
        startTime = TimeAdd(endTime, "-"+FieldValue("duration")+" hours")
    end if
     
    ' Only if this entry is flagged as publishable
    if FieldValue("publish") = "1" then
        Log("====================")
        Log("jobId: " + jobId)
        Log("dateTime: " + entryDateTime)
        Log("startTime: " + startTime)
        Log("endTime: " + endTime)
        Log("comments: " + comments)
        Log("duration (raw): " + FieldValue("duration"))
        Log("duration (formatted): " + duration)

        if Trim(jobId) = "" then
            Log("Could not get JobID for activity " + FieldValue("activityId") + " entry '" + comments + "'")
            Log("Cannot push this entry: date=" + entryDateTime + "; duration = " + duration + "; comments=" + comments)
            activityErrors = activityErrors + "Could not get JobID for activity " + FieldValue("activityId") + EOL
            publishResult = -2
        else
            
            dim entry as integer = DictionaryCreate(kAny)
            
            if Trim(entryDate) = "" then
                MsgBox("Could not calculate date for entry: " + FieldValue("entry"))
                Log("Aborting because could not calculate date for an entry")
                ScriptComplete(-1)
                return
            end if
            DictSetStr(entry, "BreakTime", "0.00")
            DictSetStr(entry, "Date", entryDate)
            DictSetStr(entry, "JobID", DictionaryGetString(kJobs, jobId))
            DictSetStr(entry, "Comment", comments)
            DictSetStr(entry, "Hours", duration)
            DictSetStr(entry, "ISOStartTime", startTime)
            DictSetStr(entry, "ISOEndTime", endTime)
            DictSetStr(entry, "TaskID", pTaskId)
            DictSetDict(kExportEntries, entryDateTime, entry)
            ' The kExportEntries dictionary will retain a reference to this entry dictionary,
            ' so we should close ours.
            DictionaryClose(entry)
        end if
    else
        Log("====================")
        Log("SKIP (activity set to not be published):")
        Log("jobId: " + jobId)
        Log("dateTime: " + entryDateTime)
        Log("startTime: " + startTime)
        Log("endTime: " + endTime)
        Log("comments: " + comments)
    end if

    call NextRecord()
next

if publishResult = -2 then
    MsgBox "Publish aborted due to errors with activity records:" + EOL + EOL + activityErrors
    ScriptComplete(-4)
    return
end if

' Get current entries for the selected week.
' Input() for the Rest device defaults to a GET request unless specified otherwise.
' Prompt for Input() for the Rest device is the URL to perform a request against,
' which is prefixed with the resource setting value specified when opening the device
' (see "BaseURL" above).
dim currentEntriesJson as string
currentEntriesJson = HttpDoGetRequest(kHttp, "Companies/" + pCompanyId + "/Users/" + pUserId + _
					     "/TimeEntries?startdate=" + pStartDate.ReplaceAll("-","") + _
					     "&enddate=" + pEndDate.ReplaceAll("-",""), 10)

' Json types:
' JsonType(0) --- return type code for data that Json object contains
' JsonIsArray(0) --- shorthand to check if it's an array
' JsonIsObject(0) --- ditto for objects

' Json convert to dictionary approach:
' JsonGetDictionary(0) --- turn Json object into a dictionary if that's what it is
' JsonGetDictionaryAt(0, i)

' Json arrays:
' xx JsonGetArray(0) -- doesn't exist, Json object IS the array
' JsonGetArrayAt(0, i) --- makes more sense, it's just another Json number (array nested in an array)
' JsonCount(0) --- count of items in an array or dictionary

' Maybe instead of trying to convert Json objects to Dictionaries (which internally they are not),
' just access their contents directly:
' JsonGetElementAt(0, i) --- returns Json number (use helpers, see below)
' JsonGetStringAt(0, i)
' JsonSetStringAt(0, i, s)
' JsonSetElementAt(0, i, n) --- store a nested Json item within a parent Json array
' JsonTypeAt(0, i)
' JsonElementType(0, key)
' JsonGetElement(0, key) --- returns a Json number which can then be used to get whatever type it is
'	--- HOWEVER this returns a Json obj number should be closed so we have helper functions:
' JsonGetString(0, key) --- return a string directly
' JsonGetNumber(0, key)
' JsonSetElement(0, key, n) --- store nested Json item within a parent json object


' Figure out what entries need to be added by comparing to what is in the API currently
' Remove entries from the list that have already been added
call JsonParse(kApiResult, currentEntriesJson)
' If valid, loop over the result
if JsonIsArray(kApiResult) then
	'call IterCreateOverJson(kIter, kApiResult)
	for jsonIdx as integer = 0 to JsonCount(kApiResult)-1
		' Remember to close any objects you get from an Iter -- these numbers are for the benefit
		' of this script only and keeping them open can eat up memeory.
		dim row as integer = JsonGetElementAt(kApiResult, jsonIdx)
		dim entries as integer = JsonGetElement(row, "TimeEntries")
		for entryIdx as integer = 0 to JsonCount(entries)-1
			dim entry as integer = JsonGetElementAt(entries, entryIdx)
			dim comment as string = JsonGetString(entry, "Comment") 
			dim remove as boolean = false
			for exportIdx as integer = 0 to DictionaryCount(kExportEntries)-1
				dim exportEntry as integer = DictGetDictAt(kExportEntries, exportIdx)
				if DictGetStr(exportEntry, "Comment") = comment then
					Log("Entry already exists, removing from insert list: " + comment)
					DictionaryRemoveAt(kExportEntries, exportIdx)
					exit
				end if
				DictionaryClose(exportEntry)
			next
			JsonClose(entry)
		next
		JsonClose(entries)
		JsonClose(row)
	next	
	JsonClose(kApiResult)
else
	MsgBox("Invalid response from ClickTime: " + EOL + EOL + currentEntriesJson)
	JsonClose(kApiResult)
    ScriptComplete(-1)
	return
end if

Log("Pushing " + str(DictionaryCount(kExportEntries)) + " entries to ClickTime...")

' Push entries that are left to API
for exportIdx as integer = 0 to DictionaryCount(kExportEntries)-1
	dim exportRow as integer = DictGetDictAt(kExportEntries, exportIdx)
	dim exportJson as integer = DictionaryToJson(exportRow)
	'JsonSetString(exportJson, "BreakTime", DictGetStr(exportRow, "BreakTime"))
	'JsonSetString(exportJson, "JobID", DictGetStr(exportRow, "JobID"))
	'JsonSetString(exportJson, "TaskID", DictGetStr(exportRow, "TaskID"))
	'JsonSetString(exportJson, "Comment", DictGetStr(exportRow, "Comment"))
	'JsonSetString(exportJson, "Hours", DictGetStr(exportRow, "Hours"))
	dim data as string = JsonToString(exportJson)
	Log("Post Data: " + data)
	HttpSetRequestHeader(kHttp, "Content-Length", Str(Len(data)))
	dim apiResult as string = HttpDoPostRequest(kHttp, "Companies/" + pCompanyId + "/Users/" + pUserId + _
	                       	     			 "/TimeEntries", 10, _
							 "application/json", _
							 data)
	Log("API Result: " + apiResult)
	if apiResult = "" then
		MsgBox("Bad response from ClickTime trying to create entry:" + EOL + EOL + data)
		Log("Aborting further requests since this one failed")
        ScriptComplete(-1)
		return
    else
        if left(apiResult, 1) = "{" then
            call JsonParse(kApiResult, apiResult)
            dim statusCode as integer = JsonGetNumber(kAPiResult, "Status")
            if statusCode <> 200 then
                dim statusDetail as string = JsonGetString(kApiResult, "Detail")
                if statusDetail = "" then
                    statusDetail = "Invalid status code returned from API " + Str(statusCode)
                end if
                MsgBox("API Error" + EOL + EOL + statusDetail)
                JsonClose(kApiResult)
                ScriptComplete(-3)
                return
            end if
        end if
	end if
	DictionaryClose(exportRow)
	JsonClose(exportJson)
next


' Create list of entries that are in API but not present locally
' Show list of potentially surplus entries to user

' Cleanup all objects
HttpClose(kHttp)
DictionaryClose(kExportEntries)	' Closes all child objects if they have not been "Opened"
DictionaryClose(kJobs)

ScriptComplete(publishResult)

